#include <stdio.h>
#include <libfawk/strcasecmp_hack.h>

/* Glue: the host application implements this to report errors from the parser
   or execution */
FAWK_API void libfawk_error(fawk_ctx_t *ctx, const char *str, const char *loc_fn, long loc_line, long loc_col);

/* Common case shorthand for returning a value as well */
#define LIBFAWK_ERROR(ctx, str, loc_fn, loc_line, loc_col, retval) \
	do { libfawk_error(ctx, str, loc_fn, loc_line, loc_col); return retval; } while(0)

/*** The rest of this file contains internal API for parsers ***/

FAWK_API void fawk_errbuff(fawk_ctx_t *ctx, size_t len);
FAWK_API void fawk_close_include(fawk_ctx_t *ctx, fawk_src_t *src);

/* FAWKI_INCDEC parameter bits */
#define FAWK_INCDEC_INC 1
#define FAWK_INCDEC_POST 2

#define FAWK_ERR ctx->errbuff
#define FAWK_ERROR(ctx, len, fmt) \
do { \
	fawk_errbuff(ctx, len); \
	if (ctx->errbuff != NULL) { sprintf fmt; libfawk_error(ctx, ctx->errbuff, "<runtime>", ctx->code.code[ctx->ip].line, 0); } \
	ctx->exec.error = 1; \
} while(0)

/* common code for jumping back from the end of a pre-test loop */
#define loop_pretest_jumpback() \
			size_t skip, back; \
			skip = fawk_pop_num(ctx, 1); \
			back = fawk_pop_num(ctx, 1); \
			fawkc_addi(ctx, FAWKI_JMP); \
			fawkc_addnum(ctx, back); \
			ctx->code.code[skip].data.num = FAWK_CURR_IP();

/* need to fetch SUBSEP on the fly */
#define parse_aix_expr() \
			fawkc_addi(ctx, FAWKI_MAKE_SYMREF); \
			fawkc_addsymref(ctx, "SUBSEP", ctx->compiler.numidx, 0); \
			fawkc_addnum(ctx, 0); \
			fawkc_addi(ctx, FAWKI_PUSH_SYMVAL); \
			fawkc_addi(ctx, FAWKI_CONCAT);

#define fawk_parser_loop(yyctxtype, STYPE, lex, parse, ctx, next, done) \
	yyctxtype yyctx; \
	int res; \
	parse ## _init(&yyctx); \
	for(;;) { \
		STYPE lval; \
		res = parse(&yyctx, ctx, lex(&lval, ctx), &lval); \
		if (res != next) break; \
	} \
	return res != done;
